@@ -5,6 +5,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
+
 import random
 
 from typing import (
@@ -927,7 +929,7 @@ class UpgradeOperation(BaseOperation):
 
     def has_upgrade_action(self, name):
         """Check whether the upgrade operation will perform this action"""
-        return name in self._upgrade_actions_names
+        return name in self.upgrade_actions_names
 
     def print_post_op_messages(self):
         """print post upgrade operation warning messages"""
