@@ -1,5 +1,5 @@
 pub mod dirstate_map;
-pub mod dispatch;
 pub mod on_disk;
+pub mod owning;
 pub mod path_with_basename;
 pub mod status;
