@@ -35,7 +35,9 @@ pub mod config;
 pub mod lock;
 pub mod logging;
 pub mod operations;
+pub mod progress;
 pub mod revset;
+pub mod update;
 pub mod utils;
 pub mod vfs;
 
@@ -44,10 +46,14 @@ pub use filepatterns::{
     parse_pattern_syntax_kind, read_pattern_file, IgnorePattern,
     PatternFileWarning, PatternSyntax,
 };
-use std::collections::HashMap;
 use std::fmt;
+use std::{collections::HashMap, sync::atomic::AtomicBool};
 use twox_hash::RandomXxHashBuilder64;
 
+/// Used to communicate with threads spawned from code within this crate that
+/// they should stop their work (SIGINT was received).
+pub static INTERRUPT_RECEIVED: AtomicBool = AtomicBool::new(false);
+
 pub type LineNumber = usize;
 
 /// Rust's default hasher is too slow because it tries to prevent collision
