@@ -17,6 +17,8 @@ echo b > d/b
 ln -s d dlink
 ln -s d dlink2
 ln -s d dlink3
+mkdir d2
+echo a > d2/a
 cd ..
 svn add *
 svn ci -m 'initial'
@@ -50,6 +52,30 @@ EOF
 
 python $RSVN --message=clobber2 --username=evil `pwd`/repo < clobber.rsvn
 
+# Create d2 in branch so d2 has 'a' is in branch/d2 and trunk/d2,
+# 'b' is in trunk/d2 and 'c' is in branch/d2
+cd wc/trunk
+echo b > d2/b
+svn add d2/b
+svn ci -m adddb
+cd ..
+svn up
+svn cp trunk branches/branch
+cd branches/branch
+svn rm d2/b
+echo c > d2/c
+svn add d2/c
+cd ../..
+svn ci -m branch
+svn up
+cd ..
+
+cat > clobber.rsvn <<EOF
+rdelete trunk/d2
+rcopy branches/branch/d2 trunk/d2
+EOF
+python $RSVN --message=clobberdir --username=evil `pwd`/repo < clobber.rsvn
+
 svn log -v file://`pwd`/repo
 
 svnadmin dump repo > ../replace.svndump
