@@ -1,11 +1,11 @@
-from __future__ import absolute_import, print_function
-
 # Import something from Mercurial, so the module loader gets initialized.
 from mercurial import pycompat
+
 del pycompat  # unused for now
 
 from hgext.lfs import pointer
 
+
 def tryparse(text):
     r = {}
     try:
@@ -19,11 +19,14 @@ def tryparse(text):
             print('reconstructed text differs')
     return r
 
-t = (b'version https://git-lfs.github.com/spec/v1\n'
-     b'oid sha256:4d7a214614ab2935c943f9e0ff69d22eadbb8f32b1'
-     b'258daaa5e2ca24d17e2393\n'
-     b'size 12345\n'
-     b'x-foo extra-information\n')
+
+t = (
+    b'version https://git-lfs.github.com/spec/v1\n'
+    b'oid sha256:4d7a214614ab2935c943f9e0ff69d22eadbb8f32b1'
+    b'258daaa5e2ca24d17e2393\n'
+    b'size 12345\n'
+    b'x-foo extra-information\n'
+)
 
 tryparse(b'')
 tryparse(t)
